import argparse
import math
import os
import re

import pandas as pd
from sparc.curation.tools.definitions import SIZE_NAME


def is_samefile(loc1, loc2):
    if pd.notnull(loc2):
        if os.path.isfile(loc2):
            return os.path.samefile(loc1, loc2)

    return False


def convert_size(size_bytes):
    if size_bytes == 0:
        return "0B"
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return f"{s}{SIZE_NAME[i]}"


def convert_to_bytes(size_string):
    m = re.match(r'^(\d+)(B|KiB|MiB|GiB|PiB|EiB|ZiB|YiB)$', size_string)
    if not m:
        raise argparse.ArgumentTypeError("'" + size_string + "' is not a valid size. Expected forms like '5MiB', '3KiB', '400B'.")
    start = m.group(1)
    end = m.group(2)
    return int(start) * math.pow(1024, SIZE_NAME.index(end))


def is_same_file(path1, path2):
    """Test if path1 is the same as path2.  If stat() on either fails and the paths
     are non-empty test if the strings are the same."""
    try:
        return os.path.samefile(path1, path2)
    except FileNotFoundError:
        if path1 and path2:
            return path1 == path2

    return False



import re
import argparse
import json

VERSION = '1.2.0'
AVAILABLE_PLOT_TYPES = ['heatmap', 'timeseries']
AVAILABLE_DELIMITERS = ['tab', 'comma']


def parse_num_list(string):
    m = re.match(r'^(\d+)(?:-(\d+))?$', string)
    if not m:
        raise argparse.ArgumentTypeError("'" + string + "' is not a range of numbers. Expected forms like '0-5'.")
    start = m.group(1)
    end = m.group(2) or start
    return list(range(int(start, 10), int(end, 10) + 1))


def flatten_nested_list(nested_list):
    flat_list = []
    # Iterate over all the elements in given list
    for elem in nested_list:
        # Check if type of element is list
        if isinstance(elem, list):
            # Extend the flat list by adding contents of this element (list)
            flat_list.extend(flatten_nested_list(elem))
        else:
            # Append the element to the list
            flat_list.append(elem)
    return flat_list


def main():
    parser = argparse.ArgumentParser(description='Create an annotation for a SPARC plot. '
                                                 'The Y_AXES_COLUMNS can either be single numbers or a range in the form 5-8. '
                                                 'The start and end numbers are included in the range. '
                                                 'The -y/--y-axes-columns argument will consume the positional plot type argument. '
                                                 'That means the positional argument cannot follow the -y/--y-axes-columns.')
    parser.add_argument("plot_type", help='must define a plot type which is one of; ' + ', '.join(AVAILABLE_PLOT_TYPES) + '.',
                        choices=AVAILABLE_PLOT_TYPES)
    parser.add_argument("-x", "--x-axis-column", help="integer index for the independent column (zero based). Default is 0.",
                        type=int, default=0)
    parser.add_argument("-y", "--y-axes-columns", help="list of indices for the dependent columns (zero based). Can be used multiple times."
                                                       " Can be specified as a range e.g. 5-8. Default is [].",
                        default=[], nargs='*', action="append", type=parse_num_list)
    parser.add_argument("-n", "--no-header", help="Boolean to indicate whether a header line is missing. Default is False.",
                        action="store_true", default=False)
    parser.add_argument("-r", "--row-major", help="Boolean to indicate whether the data is row major or column major. Default is False.",
                        action="store_true", default=False)
    parser.add_argument("-d", "--delimiter", help="The type of delimiter used, must be one of; " + ", ".join(AVAILABLE_DELIMITERS) + ". Default is comma.",
                        default='comma', choices=AVAILABLE_DELIMITERS)

    args = parser.parse_args()
    attrs = {
        'style': args.plot_type,
    }
    if args.x_axis_column != 0:
        attrs['x-axis'] = args.x_axis_column

    if args.delimiter != 'comma':
        attrs['delimiter'] = args.delimiter

    if len(args.y_axes_columns):
        attrs['y-axes-columns'] = flatten_nested_list(args.y_axes_columns)

    if args.no_header:
        attrs['no-header'] = args.no_header

    if args.row_major:
        attrs['row-major'] = args.row_major

    data = {
        'version': VERSION,
        'type': 'plot',
        'attrs': attrs
    }
    print(json.dumps(data))


main()



import json
import os
from pathlib import Path

from sparc.curation.tools.base import Singleton


ZINC_GRAPHICS_TYPES = ["points", "lines", "surfaces", "contours", "streamlines"]


def _is_graphics_entry(entry):
    if 'URL' in entry and 'Type' in entry:
        entry_type = entry['Type']
        if entry_type.lower() in ZINC_GRAPHICS_TYPES:
            return True

    return False


def _is_view_entry(entry):
    if 'URL' in entry and 'Type' in entry:
        entry_type = entry['Type']
        if entry_type.lower() == "view":
            return True

    return False


def test_for_metadata(json_data):
    have_viewable_graphics = False
    have_view_reference = False
    if json_data:
        if isinstance(json_data, list):
            for entry in json_data:
                if not have_viewable_graphics and _is_graphics_entry(entry):
                    have_viewable_graphics = True
                if not have_view_reference and _is_view_entry(entry):
                    have_view_reference = True

    return have_view_reference and have_viewable_graphics


def test_for_view(json_data):
    is_view = False
    if json_data:
        if isinstance(json_data, dict):
            expected_keys = ["farPlane", "nearPlane", "upVector", "targetPosition", "eyePosition"]
            missing_key = False
            for expected_key in expected_keys:
                if expected_key not in json_data:
                    missing_key = True

            is_view = not missing_key

    return is_view


def is_json_of_type(r, max_size, test_func):
    result = False
    if os.path.getsize(r) < max_size and os.path.isfile(r):
        try:
            with open(r, encoding='utf-8') as f:
                file_data = f.read()
        except UnicodeDecodeError:
            return result
        except IsADirectoryError:
            return result

        try:
            data = json.loads(file_data)
            result = test_func(data)
        except json.decoder.JSONDecodeError:
            return result

    return result


def search_for_metadata_files(dataset_dir, max_size):
    metadata = []
    result = list(Path(dataset_dir).rglob("*"))
    for r in result:
        meta = is_json_of_type(r, max_size, test_for_metadata)

        if meta:
            metadata.append(str(r))

    return metadata


def search_for_thumbnail_files(dataset_dir):
    result = list(Path(dataset_dir).rglob("*thumbnail*"))
    list(Path(dataset_dir).rglob("*.png"))
    list(Path(dataset_dir).rglob("*.jpeg"))
    list(Path(dataset_dir).rglob("*.jpg"))
    # For each result:
    #   - Is this file actually an image?
    # Probably just leave this for now and go with the simple name comparison.
    return [str(x) for x in result]


def search_for_view_files(dataset_dir, max_size):
    metadata = []
    result = list(Path(dataset_dir).rglob("*"))
    for r in result:
        meta = is_json_of_type(r, max_size, test_for_view)

        if meta:
            metadata.append(str(r))

    return metadata


class OnDiskFiles(metaclass=Singleton):
    # dataFrame_dir = ""
    _onDiskFiles = None
    _scaffold = None

    class Scaffold(object):
        _scaffold_files = {
            'metadata': [],
            'view': [],
            'thumbnail': [],
        }

        def set_metadate_files(self, files):
            self._scaffold_files['metadata'] = files

        def get_metadata_files(self):
            return self._scaffold_files['metadata']

        def set_view_files(self, files):
            self._scaffold_files['view'] = files

        def get_view_files(self):
            return self._scaffold_files['view']

        def set_thumbnail_files(self, files):
            self._scaffold_files['thumbnail'] = files

        def get_thumbnail_files(self):
            return self._scaffold_files['thumbnail']

    def get_scaffold_data(self):
        return self._scaffold

    def setup_dataset(self, dataset_dir, max_size):
        self._scaffold = OnDiskFiles.Scaffold()
        self._scaffold.set_metadate_files(search_for_metadata_files(dataset_dir, max_size))
        self._scaffold.set_view_files(search_for_view_files(dataset_dir, max_size))
        self._scaffold.set_thumbnail_files(search_for_thumbnail_files(dataset_dir))
        return self



import argparse
import os
import pandas as pd

from sparc.curation.tools.annotations.scaffold import IncorrectAnnotationError, NoViewError, NoThumbnailError, NoDerivedFromError, NotAnnotatedError
from sparc.curation.tools.definitions import ADDITIONAL_TYPES_COLUMN, SOURCE_OF_COLUMN, \
    DERIVED_FROM_COLUMN, SCAFFOLD_VIEW_MIME, SCAFFOLD_THUMBNAIL_MIME, SCAFFOLD_FILE_MIME
from sparc.curation.tools.errors import AnnotationDirectoryNoWriteAccess
from sparc.curation.tools.manifests import ManifestDataFrame
from sparc.curation.tools.ondisk import OnDiskFiles
from sparc.curation.tools.utilities import convert_to_bytes, is_same_file


def check_additional_types_annotations():
    errors = []
    errors += ManifestDataFrame().get_scaffold_data().get_missing_annotations(OnDiskFiles())
    errors += ManifestDataFrame().get_scaffold_data().get_incorrect_annotations(OnDiskFiles())
    return errors


def check_scaffold_view_annotations():
    errors = []
    errors.extend(ManifestDataFrame().get_scaffold_data().get_scaffold_no_view(OnDiskFiles()))
    errors.extend(ManifestDataFrame().get_scaffold_data().get_view_no_scaffold(OnDiskFiles()))
    return errors


def check_scaffold_thumbnail_annotations():
    errors = []
    errors.extend(ManifestDataFrame().get_scaffold_data().get_view_no_thumbnail(OnDiskFiles()))
    errors.extend(ManifestDataFrame().get_scaffold_data().get_thumbnail_no_view(OnDiskFiles()))
    return errors


def get_errors():
    errors = []
    errors.extend(check_additional_types_annotations())
    errors.extend(check_scaffold_view_annotations())
    errors.extend(check_scaffold_thumbnail_annotations())
    return errors


def get_confirmation_message(error):
    """
    "To fix this error, the 'additional types' of 'filename' in 'manifestFile' will be set to 'MIME'."
    "To fix this error, a manifestFile will be created under manifestDir, and will insert the filename in this manifestFile with 'additional types' MIME."

    "To fix this error, the data of filename in manifestFile will be deleted."
    # TODO or NOT TODO: return different message based on input error type
    """
    message = "Let this magic tool fix this error for you."
    return message


def set_source_of_column(file_location, mime):
    manifestDataFrame = ManifestDataFrame().get_manifest()
    # fileDir = os.path.dirname(file_location)
    fileName = os.path.basename(file_location)
    # Before add view, the scaffold metadata must already been annotated in manifest, otherwise fix the NoAnnotatedError first
    fileDF = manifestDataFrame[manifestDataFrame["filename"].str.contains(r'\(/|\)*' + fileName)]
    manifestDir = fileDF["manifest_dir"].iloc[0]
    # Search thumbnail in dataframe with same manifest_dir as scafflod
    # If found, set it as isSourceOf
    # If not, search file 

    mDF = pd.read_excel(os.path.join(manifestDir, "manifest.xlsx"))
    if SOURCE_OF_COLUMN not in mDF.columns:
        mDF[SOURCE_OF_COLUMN] = ""
    # TODO Change to Views
    viewNames = mDF["filename"][mDF["additional types"] == mime]

    if viewNames.empty:
        # Search from files
        viewLocations = []
        if mime == SCAFFOLD_VIEW_MIME:
            viewLocations = OnDiskFiles().get_scaffold_data().get_view_files()
        elif mime == SCAFFOLD_THUMBNAIL_MIME:
            viewLocations = OnDiskFiles().get_scaffold_data().get_thumbnail_files()
        viewNames = [os.path.relpath(view, manifestDir) for view in viewLocations]

    mDF.loc[mDF["filename"].str.contains(r'\(/|\)*' + fileName), SOURCE_OF_COLUMN] = ','.join(viewNames)
    # else:
    # Find the manifest file contain the file annotation
    # mDF = pd.read_excel(io)
    # TODO
    mDF.to_excel(os.path.join(manifestDir, "manifest.xlsx"), index=False, header=True)


def update_derived_from(file_location, mime):
    manifestDataFrame = ManifestDataFrame().get_manifest()
    # fileDir = os.path.dirname(file_location)
    fileName = os.path.basename(file_location)
    # Before add view, the scaffold metadata must already been annotated in manifest, otherwise fix the NoAnnotatedError first
    fileDF = manifestDataFrame[manifestDataFrame["filename"].str.contains(r'\(/|\)*' + fileName)]
    manifestDir = fileDF["manifest_dir"].iloc[0]
    # Search thumbnail in dataframe with same manifest_dir as scafflod
    # If found, set it as isSourceOf
    # If not, search file 
    mDF = pd.read_excel(os.path.join(manifestDir, "manifest.xlsx"))
    if DERIVED_FROM_COLUMN not in mDF.columns:
        mDF[DERIVED_FROM_COLUMN] = ""
    # TODO Change to Views
    parentMime = SCAFFOLD_VIEW_MIME
    if mime == SCAFFOLD_VIEW_MIME:
        parentMime = SCAFFOLD_FILE_MIME
    viewNames = mDF["filename"][mDF[ADDITIONAL_TYPES_COLUMN] == parentMime]

    if viewNames.empty:
        # Search from files
        viewLocations = []
        if parentMime == SCAFFOLD_VIEW_MIME:
            viewLocations = OnDiskFiles().get_scaffold_data().get_view_files()
        elif parentMime == SCAFFOLD_FILE_MIME:
            viewLocations = OnDiskFiles().get_scaffold_data().get_metadata_files()
        viewNames = [os.path.relpath(view, manifestDir) for view in viewLocations]

    mDF.loc[mDF["filename"].str.contains(r'\(/|\)*' + fileName), DERIVED_FROM_COLUMN] = ','.join(viewNames)
    # else:
    # Find the manifest file contain the file annotation
    # mDF = pd.read_excel(io)
    # TODO
    mDF.to_excel(os.path.join(manifestDir, "manifest.xlsx"), index=False, header=True)


def update_additional_type(file_location, file_mime):
    # TODO try not read all the manifest again
    manifestDataFrame = ManifestDataFrame().get_manifest()
    fileDir = os.path.dirname(file_location)
    fileName = os.path.basename(file_location)
    fileDF = manifestDataFrame[manifestDataFrame["filename"].str.contains(r'\(/|\)*' + fileName)]
    # If fileDF is empty, means there's no manifest file contain this file.
    # Check if there's manifest file under same dir. Add file to the manifest.
    # If no manifest file create new manifest file
    if fileDF.empty:
        # Check if there's manifest file under Scaffold File Dir
        newRow = pd.DataFrame({"filename": fileName, "additional types": file_mime}, index=[1])
        if not manifestDataFrame[manifestDataFrame["manifest_dir"] == fileDir].empty:
            mDF = pd.read_excel(os.path.join(fileDir, "manifest.xlsx"))
            newRow = mDF.append(newRow, ignore_index=True)
        newRow.to_excel(os.path.join(fileDir, "manifest.xlsx"), index=False, header=True)

    for index, row in fileDF.iterrows():
        fileLocation = os.path.join(row["manifest_dir"], row['filename'])
        if is_same_file(file_location, fileLocation):
            mDF = pd.read_excel(os.path.join(row["manifest_dir"], "manifest.xlsx"), sheet_name=row["sheet_name"])
            if ADDITIONAL_TYPES_COLUMN not in mDF.columns:
                mDF[ADDITIONAL_TYPES_COLUMN] = ""
            mDF.loc[mDF["filename"] == row['filename'], ADDITIONAL_TYPES_COLUMN] = file_mime
            mDF.to_excel(os.path.join(row["manifest_dir"], "manifest.xlsx"), sheet_name=row["sheet_name"], index=False, header=True)


def fix_error(error):
    checked_locations = []
    for manifest_dir in ManifestDataFrame().get_manifest()['manifest_dir']:
        if manifest_dir not in checked_locations:
            checked_locations.append(manifest_dir)
            if not os.access(manifest_dir, os.W_OK):
                raise AnnotationDirectoryNoWriteAccess(f"Cannot write to directory {manifest_dir}.")

    # Check incorrect annotation before no annotation
    if isinstance(error, IncorrectAnnotationError):
        update_additional_type(error.get_location(), None)
    elif isinstance(error, NotAnnotatedError):
        update_additional_type(error.get_location(), error.get_mime())
    elif isinstance(error, NoViewError):
        set_source_of_column(error.get_location(), SCAFFOLD_VIEW_MIME)
    elif isinstance(error, NoThumbnailError):
        set_source_of_column(error.get_location(), SCAFFOLD_THUMBNAIL_MIME)
    elif isinstance(error, NoDerivedFromError):
        update_derived_from(error.get_location(), error.get_mime())


def main():
    parser = argparse.ArgumentParser(description='Check scaffold annotations for a SPARC dataset.')
    parser.add_argument("dataset_dir", help='directory to check.')
    parser.add_argument("-m", "--max-size", help="Set the max size for metadata file. Default is 2MiB", default='2MiB', type=convert_to_bytes)
    parser.add_argument("-r", "--report", help="Report any errors that were found.", action='store_true')
    parser.add_argument("-f", "--fix", help="Fix any errors that were found.", action='store_true')

    args = parser.parse_args()
    dataset_dir = args.dataset_dir
    max_size = args.max_size

    # Step 1: Look at all the files in the dataset
    #   - Try to find files that I think are scaffold metadata files.
    #   - Try to find files that I think are scaffold view files.
    #   - Try ...
    OnDiskFiles().setup_dataset(dataset_dir, max_size)

    # Step 2: Read all the manifest files in the dataset
    #   - Get all the files annotated as scaffold metadata files.
    #   - Get all the files annotated as scaffold view files.
    #   - Get all the files annotated as scaffold view thumbnails.
    ManifestDataFrame().setup_dataframe(dataset_dir)

    # Step 3:
    #   - Compare the results from steps 1 and 2 and determine if they have any differences.
    #   - Problems I must look out for:
    #     - Entry in manifest file doesn't refer to an existing file.
    #     - Scaffold files I find in the dataset do not have a matching entry in a manifest.
    #     - All scaffold metadata files must have at least one view associated with it (and vice versa).
    #     - All scaffold view files should(must) have exactly one thumbnail associated with it (and vice versa).
    errors = get_errors()

    # Step 4:
    #   - Report an differences from step 1 and 2.
    if args.report:
        for error in errors:
            print(error.get_error_message())

    # Step 5:
    #   - Fix errors as identified by user.
    if args.fix:
        for error in errors:
            fix_error(error)


if __name__ == "__main__":
    main()



import os
from pathlib import Path

import pandas as pd
from sparc.curation.tools.annotations.scaffold import ScaffoldAnnotation, IncorrectAnnotationError, NotAnnotatedError, NoViewError, NoDerivedFromError, NoThumbnailError

from sparc.curation.tools.base import Singleton
from sparc.curation.tools.definitions import FILE_LOCATION_COLUMN, FILENAME_COLUMN, ADDITIONAL_TYPES_COLUMN, SCAFFOLD_FILE_MIME, SCAFFOLD_VIEW_MIME, SCAFFOLD_THUMBNAIL_MIME


class ManifestDataFrame(metaclass=Singleton):
    # dataFrame_dir = ""
    _manifestDataFrame = None
    _scaffold_data = None

    def setup_dataframe(self, dataset_dir):
        self.read_manifest(dataset_dir)
        self.setup_data()
        return self

    def read_manifest(self, dataset_dir):
        # self._annotatedFileList = None
        # self._realScaffoldList = None
        # self.dataFrame_dir = dataset_dir
        result = list(Path(dataset_dir).rglob("manifest.xlsx"))
        self._manifestDataFrame = pd.DataFrame()
        for r in result:
            xl_file = pd.ExcelFile(r)
            # print(xl_file)
            for sheet_name in xl_file.sheet_names:
                currentDataFrame = xl_file.parse(sheet_name)
                currentDataFrame['sheet_name'] = sheet_name
                currentDataFrame['manifest_dir'] = os.path.dirname(r)
                self._manifestDataFrame = pd.concat([currentDataFrame, self._manifestDataFrame])
        # print(manifestDataFrame)
        self._manifestDataFrame[FILE_LOCATION_COLUMN] = self._manifestDataFrame.apply(
            lambda row: os.path.join(row['manifest_dir'], row[FILENAME_COLUMN]) if pd.notnull(row[FILENAME_COLUMN]) else None, axis=1)
        return self._manifestDataFrame

    def get_manifest(self):
        return self._manifestDataFrame

    def setup_data(self):
        self._scaffold_data = ManifestDataFrame.Scaffold()
        self._scaffold_data.set_scaffold_annotations(
            [ScaffoldAnnotation(row) for i, row in self._manifestDataFrame[self._manifestDataFrame[ADDITIONAL_TYPES_COLUMN].notnull()].iterrows()]
        )
        self._scaffold_data.set_scaffold_locations([i.get_location() for i in self._scaffold_data.get_scaffold_annotations()])

    def get_scaffold_data(self):
        return self._scaffold_data

    class Scaffold(object):
        _data = {
            'annotations': [],
            'locations': [],
        }

        def set_scaffold_annotations(self, annotations):
            self._data['annotations'] = annotations

        def get_scaffold_annotations(self):
            return self._data['annotations']

        def set_scaffold_locations(self, locations):
            self._data['locations'] = locations

        def get_scaffold_locations(self):
            return self._data['locations']

        def get_metadata_filenames(self):
            filenames = []
            for i in self._data['annotations']:
                if i.get_additional_type() == SCAFFOLD_FILE_MIME:
                    filenames.append(i.get_location())

            return filenames

        def get_derived_filenames(self, source):
            for i in self._data['annotations']:
                if i.get_parent() == source:
                    return i.get_location()

            return ''

        def get_incorrect_annotations(self, on_disk):
            errors = []

            on_disk_metadata_files = on_disk.get_scaffold_data().get_metadata_files()
            on_disk_view_files = on_disk.get_scaffold_data().get_view_files()
            on_disk_thumbnail_files = on_disk.get_scaffold_data().get_thumbnail_files()

            for i in self._data['annotations']:
                if i.get_additional_type() == SCAFFOLD_FILE_MIME:
                    if i.get_location() not in on_disk_metadata_files:
                        errors.append(IncorrectAnnotationError(i.get_location(), i.get_additional_type()))

                if i.get_additional_type() == SCAFFOLD_VIEW_MIME:
                    if i.get_location() not in on_disk_view_files:
                        errors.append(IncorrectAnnotationError(i.get_location(), i.get_additional_type()))

                if i.get_additional_type() == SCAFFOLD_THUMBNAIL_MIME:
                    if i.get_location() not in on_disk_thumbnail_files:
                        errors.append(IncorrectAnnotationError(i.get_location(), i.get_additional_type()))
            return errors

        def get_missing_annotations(self, on_disk):
            errors = []

            on_disk_metadata_files = on_disk.get_scaffold_data().get_metadata_files()
            on_disk_view_files = on_disk.get_scaffold_data().get_view_files()
            on_disk_thumbnail_files = on_disk.get_scaffold_data().get_thumbnail_files()

            for i in on_disk_metadata_files:
                if i not in self._data['locations']:
                    errors.append(NotAnnotatedError(i, SCAFFOLD_FILE_MIME))

            for i in on_disk_view_files:
                if i not in self._data['locations']:
                    errors.append(NotAnnotatedError(i, SCAFFOLD_VIEW_MIME))

            for i in on_disk_thumbnail_files:
                if i not in self._data['locations']:
                    errors.append(NotAnnotatedError(i, SCAFFOLD_THUMBNAIL_MIME))

            return errors

        def get_scaffold_no_view(self, on_disk):
            errors = []

            on_disk_metadata_files = on_disk.get_scaffold_data().get_metadata_files()

            for i in self._data['annotations']:
                if i.get_location() in on_disk_metadata_files and not i.get_children():
                    errors.append(NoViewError(i.get_location()))
            return errors

        def get_view_no_scaffold(self, on_disk):
            errors = []
            on_disk_view_files = on_disk.get_scaffold_data().get_view_files()
            for i in self._data['annotations']:
                if i.get_location() in on_disk_view_files and not i.get_parent():
                    errors.append(NoDerivedFromError(i.get_location(), SCAFFOLD_VIEW_MIME))
            return errors

        def get_view_no_thumbnail(self, on_disk):
            result = []
            on_disk_view_files = on_disk.get_scaffold_data().get_view_files()
            for i in self._data['annotations']:
                if i.get_location() in on_disk_view_files and not i.get_children():
                    result.append(NoThumbnailError(i.get_location()))
            return result

        def get_thumbnail_no_view(self, on_disk):
            result = []
            on_disk_thumbnail_files = on_disk.get_scaffold_data().get_thumbnail_files()
            for i in self._data['annotations']:
                if i.get_location() in on_disk_thumbnail_files and not i.get_parent():
                    result.append(NoDerivedFromError(i.get_location(), SCAFFOLD_THUMBNAIL_MIME))
            return result



# VERSION = sparc.curation.tools.__version__

SCAFFOLD_DIR_MIME = 'inode/vnd.abi.scaffold+directory'
SCAFFOLD_FILE_MIME = 'inode/vnd.abi.scaffold+file'
SCAFFOLD_VIEW_MIME = 'inode/vnd.abi.scaffold.view+file'
SCAFFOLD_THUMBNAIL_MIME = 'inode/vnd.abi.scaffold.thumbnail+file'
TARGET_MIMES = [SCAFFOLD_DIR_MIME, SCAFFOLD_FILE_MIME, SCAFFOLD_THUMBNAIL_MIME]

SIZE_NAME = ("B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB")

FILENAME_COLUMN = 'filename'
ADDITIONAL_TYPES_COLUMN = 'additional types'
MANIFEST_DIR_COLUMN = 'manifest_dir'
SOURCE_OF_COLUMN = 'isSourceOf'
DERIVED_FROM_COLUMN = 'isDerivedFrom'
FILE_LOCATION_COLUMN = 'file_location'

MIMETYPE_TO_FILETYPE_MAP = {
    SCAFFOLD_FILE_MIME: 'Metadata',
    SCAFFOLD_VIEW_MIME: 'View',
    SCAFFOLD_THUMBNAIL_MIME: 'Thumbnail'
}



import os

from sparc.curation.tools.definitions import MIMETYPE_TO_FILETYPE_MAP, FILE_LOCATION_COLUMN, MANIFEST_DIR_COLUMN, FILENAME_COLUMN, ADDITIONAL_TYPES_COLUMN, SOURCE_OF_COLUMN, \
    DERIVED_FROM_COLUMN


class ScaffoldAnnotationError(object):

    def __init__(self, message, location):
        self._message = message
        self._location = location

    def get_location(self):
        return self._location

    def get_error_message(self):
        return f'Error: {self._message}'

    # def __str__(self):
    #     return f'Error: {self._message}'


class NotAnnotatedError(ScaffoldAnnotationError):
    def __init__(self, location, mime):
        self._mime = mime
        fileType = MIMETYPE_TO_FILETYPE_MAP.get(mime, 'unknown')
        message = f"Found Scaffold '{fileType}' file that is not annotated '{location}'."
        super(NotAnnotatedError, self).__init__(message, location)

    def get_mime(self):
        return self._mime


class NoViewError(ScaffoldAnnotationError):
    def __init__(self, location):
        message = f"Found scaffold metadata file that has no view '{location}'."
        super(NoViewError, self).__init__(message, location)


class NoThumbnailError(ScaffoldAnnotationError):
    def __init__(self, location):
        message = f"Found scaffold view file that has no thumbnail '{location}'."
        super(NoThumbnailError, self).__init__(message, location)


class NoDerivedFromError(ScaffoldAnnotationError):
    def __init__(self, location, mime):
        self._mime = mime
        fileType = MIMETYPE_TO_FILETYPE_MAP.get(mime, 'unknown')
        message = f"Found '{fileType}' that has no derived from file '{location}'."
        super(NoDerivedFromError, self).__init__(message, location)

    def get_mime(self):
        return self._mime


class IncorrectAnnotationError(ScaffoldAnnotationError):
    def __init__(self, location, mime):
        self._mime = mime
        fileType = MIMETYPE_TO_FILETYPE_MAP.get(mime, 'unknown')
        message = f"File '{location}' either does not exist or is not a scaffold '{fileType}' file."
        super(IncorrectAnnotationError, self).__init__(message, location)

    def get_mime(self):
        return self._mime


class NotADirError(ScaffoldAnnotationError):
    def __init__(self, location):
        message = f"Directory '{location}' either does not exist or is not a directory."
        super(NotADirError, self).__init__(message, location)


class NotAFileError(ScaffoldAnnotationError):
    def __init__(self, location):
        message = "Not a file"
        super(NotAFileError, self).__init__(message, location)


class ScaffoldAnnotation(object):
    """
    TODO use this class to wrap one dataframe row to an object.
    Only rows with ADDITIONAL_TYPES_COLUMN will be wrapped by this class
    """

    def __init__(self, dataframe_row):
        self._dir = dataframe_row[FILE_LOCATION_COLUMN]  # This is now redundant.
        self._manifestDir = dataframe_row[MANIFEST_DIR_COLUMN]
        self._fileName = dataframe_row[FILENAME_COLUMN]
        self._location = dataframe_row[FILE_LOCATION_COLUMN]
        self._additionalType = None
        self._children = None
        self._parent = None

        if ADDITIONAL_TYPES_COLUMN in dataframe_row:
            if isinstance(dataframe_row[ADDITIONAL_TYPES_COLUMN], str):
                self._additionalType = dataframe_row[ADDITIONAL_TYPES_COLUMN]

        if SOURCE_OF_COLUMN in dataframe_row:
            if isinstance(dataframe_row[SOURCE_OF_COLUMN], str):
                self._children = [str(os.path.join(self._manifestDir, filename)) for filename in dataframe_row[SOURCE_OF_COLUMN].split(',')]

        if DERIVED_FROM_COLUMN in dataframe_row:
            if isinstance(dataframe_row[DERIVED_FROM_COLUMN], str):
                self._parent = str(os.path.join(self._manifestDir, dataframe_row[DERIVED_FROM_COLUMN]))

    def get_location(self):
        return os.path.normpath(os.path.join(self._location))

    def get_additional_type(self):
        return self._additionalType

    def set_dir(self, dir_name):
        self._dir = dir_name

    def get_dir(self):
        return self._dir

    def set_filename(self, file):
        self._fileName = file

    def get_filename(self):
        return self._fileName

    def get_children(self):
        return self._children

    def get_parent(self):
        return self._parent

    def get_thumbnail(self):
        return self._children[0]

    def __eq__(self, other):
        return os.path.samefile(self.get_location(), other.get_location())
